#define G_LOG_DOMAIN "gpod-manager"

#include "config.h"

#include "gpod-feed.h"
#include "gpod-feed-channel.h"
#include "gpod-feed-item.h"

#include "gpod-tracker.h"
#include "gpod-manager.h"

#include "gpod-library-view.h"

#include <tracker-sparql.h>

/**
 * SECTION:gpod-manager
 * @title: GpodManager
 * @short_description: manages feeds and saving of information to and from the
 *                     tracker database.
 * @include: gpod-manager.h
 *
 * The #GpodManager is object to manage the fetching, saving, and retrevial of
 * feeds and their items.
 **/

typedef struct _GpodManagerPrivate
{
    GpodTracker *tracker;
} GpodManagerPrivate;

struct _GpodManager
{
    GObject parent;
};

G_DEFINE_TYPE_WITH_PRIVATE (GpodManager, gpod_manager, G_TYPE_OBJECT)

void
gpod_manager_setting_add_feed (GpodManager *self,
                               gchar       *url)
{
    GSettings *settings = g_settings_new (PACKAGE_ID);
    GVariant *variant;
    gsize variant_size;
    gchar **feeds;

    variant = g_settings_get_value (settings, "feeds");
    variant_size = g_variant_get_size (variant);
    feeds = g_variant_dup_strv (variant, &variant_size);
    g_variant_unref (variant);

    g_print ("Feeds:\n");
    for (gsize i = 0; i < variant_size; i++)
    {
        g_print ("* %s", feeds[i]);
    }

    //g_settings_set_value (settings, "feeds", feeds);

    g_strfreev (feeds);
}

/**
 * gpod_manager_setting_get_feeds:
 * @self: a #GpodManager instance
 *
 * Returns the list of feeds the user is subscribed to, stored in GSetting.
 *
 * Returns: (gchar **) an array of strings containg all subscribed feeds, free
 *          with g_strfreev() when finished.
 */
gchar **
gpod_manager_setting_get_feeds (GpodManager *self)
{
    GSettings *settings = g_settings_new (PACKAGE_ID);
    GVariant *variant;
    gsize variant_size;
    gchar **feeds;

    variant = g_settings_get_value (settings, "feeds");
    variant_size = g_variant_get_size (variant);
    feeds = g_variant_dup_strv (variant, &variant_size);
    g_variant_unref (variant);

    return g_strdupv (feeds);
}

GList *
gpod_manager_setting_get_feeds_as_glist (GpodManager *self)
{
    gchar **feeds = gpod_manager_setting_get_feeds (self);
    GList *list = NULL;

    g_print ("%lu", sizeof (feeds));

    for (gint i = 0; i < sizeof (feeds); i++)
    {
        g_print ("%s", g_strdup (feeds[i]));
        list = g_list_append (list, g_strdup (feeds[i]));
    }

    g_strfreev (feeds);

    return list;
}

void
gpod_manager_add_feeds_to_widget_with (GpodManager *self,
                                       GtkWidget   *container,
                                       GCallback   *callback)
{

}




static void
add_feed_by_url (GpodManager *self,
                 gchar       *feed_url)
{
    GpodManagerPrivate *priv = gpod_manager_get_instance_private (self);
    GString *query = g_string_new (NULL);

    // addruri = tracker_sparql_get_uuid_urn ();
    // address = tracker_resource_new (addruri);
    //
    //
    //
    // tracker_resource_set_identifier (resource, uri);
    // text = tracker_resource_print_sparql_update (resource, NULL, NULL);
    // g_print ("%s\n", text);

    // Add to tracker
    g_string_printf (query,
                     "INSERT { "
                     "    _:FeedSettings a mfo:FeedSettings ; "
                     "        mfo:updateInterval %i . "
                     "    _:Feed a nie:DataObject, mfo:FeedChannel ; "
                     "        mfo:feedSettings _:FeedSettings ; "
                     "        nie:url <%s> "
                     "}",
                     30, feed_url);

    gpod_tracker_query_update (priv->tracker, query->str, NULL);

    g_string_free (query, TRUE);
}

static void
get_feeds ()
{

}

void
print_feed (GObject      *object,
            GAsyncResult *result,
            gpointer      user_data)
{
    TrackerSparqlCursor *cursor = NULL;
    GError *error = NULL;

    cursor = gpod_tracker_query_finish (object, result, user_data, &error);

    if (error != NULL)
    {
        g_warning ("Unable to get feed from Tracker database: %s", error->message);
        g_error_free (error);
        return;
    }

    if (!cursor)
    {
        g_print ("%s\n", "No results were found");
        return;
    }

    tracker_sparql_cursor_next (cursor, NULL, NULL);

    g_print ("Result:\n    %s\n    %s\n    %s\n    %s\n    %s\n    %s\n    %s\n\n",
             tracker_sparql_cursor_get_string (cursor, 0, NULL),
             tracker_sparql_cursor_get_string (cursor, 1, NULL),
             tracker_sparql_cursor_get_string (cursor, 2, NULL),
             tracker_sparql_cursor_get_string (cursor, 3, NULL),
             tracker_sparql_cursor_get_string (cursor, 4, NULL),
             tracker_sparql_cursor_get_string (cursor, 5, NULL),
             tracker_sparql_cursor_get_string (cursor, 6, NULL));


    //tracker_sparql_cursor_next_async (cursor, query_data->cancellable, cursor_cb, query_data);
}

void
get_feed_by_url (GpodManager *self,
                 gchar       *feed_url)
{
    GpodManagerPrivate *priv = gpod_manager_get_instance_private (self);
    GString *query = g_string_new (NULL);

    g_string_printf (query,
                     "SELECT "
                     "    ?urn AS urn"
                     "    ?title AS title "
                     "    ?description AS description "
                     "    ?copyright AS copyright "
                     "    ?updated AS updated "
                     "    ?image AS image "
                     "    ?updateInterval AS updateInterval "
                     "WHERE { "
                     "    ?urn a mfo:FeedChannel ; "
                     "        nie:url <%s> ; "
                     "        nie:title ?title ; "
                     "        nie:description ?description ; "
                     "        nie:copyright ?copyright ; "
                     "        nmo:lastMessageDate ?updated ; "
                     "        mfo:image ?image . "
                     "    OPTIONAL { "
                     "        ?urnSettings a mfo:FeedSettings ; "
                     "            mfo:updateInterval ?updateInterval . "
                     "    } "
                     "} "
                     "LIMIT 1", feed_url);

    gpod_tracker_query (priv->tracker, query->str, print_feed, NULL);

    g_string_free (query, TRUE);
}

/*
 * gpod_manager_finalize:
 * @object: An instance of #GpodManager
 *
 * Does any last minute clean-ups when the class ends.
 */
static void
gpod_manager_finalize (GObject *object)
{
    GpodManager *self = GPOD_MANAGER (object);
    GpodManagerPrivate *priv = gpod_manager_get_instance_private (self);

    g_clear_object (&priv->tracker);

    G_OBJECT_CLASS (gpod_manager_parent_class)->finalize(object);
}

/*
 * gpod_manager_init:
 * @self: An instance of #GpodManager
 *
 * Initializes various aspects of the #GpodManager. Anything the #GpodManager
 * would need before becoming presented should be done here.
 */
static void
gpod_manager_init (GpodManager *self)
{
    GpodManagerPrivate *priv = gpod_manager_get_instance_private (self);

    priv->tracker = gpod_tracker_new ();
}

/*
 * gpod_manager_class_init:
 * @klass: An instance of #GpodManagerClass
 *
 * Initializes the various virtural functions in #GpodManagerClass. In
 * addtition it connects the various #GtkWidgets from the gresource file using
 * gtk_widget_class_set_template_from_resource().
 */
static void
gpod_manager_class_init (GpodManagerClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);

    object_class->finalize = gpod_manager_finalize;
}

/**
 * gpod_manager_new:
 *
 * Creates a new #GpodManager, a custom widget to show all feeds user is
 * subscribed to.
 *
 * ## Usage
 * |[
 * GpodManager *manager;
 *
 * manager = gpod_manager_new ();
 * ]|
 *
 * Returns: #GpodManager
 */
GpodManager *
gpod_manager_new ()
{
    return g_object_new (GPOD_TYPE_MANAGER,
                         NULL);
}

void
gpod_manager_add_feed_by_url (GpodManager *self,
                              gchar       *feed_url)
{
    add_feed_by_url (self, feed_url);
}

void
gpod_manager_get_feed_by_url (GpodManager *self,
                              gchar       *feed_url)
{
    get_feed_by_url (self, feed_url);
}


GList *
gpod_manager_get_feeds (GpodManager *self)
{
    GList *feeds = NULL;

    return feeds;
}

GList *
gpod_manager_get_feed_items (GpodManager *self)
{
    GList *feeds = NULL;

    return feeds;
}
