#define G_LOG_DOMAIN "gpod-preferences"

#include "config.h"

#include "gpod-preferences.h"
#include "gpod-preferences-view.h"
#include "gpod-window.h"
#include "gpod-application.h"

#include <gtk/gtk.h>
#include <glib/gi18n.h>

/**
 * SECTION:gpod-preferences
 * @title: GpodPreferences
 * @short_description: A #GtkWindow instance created for #GpodApplication
 * @include: gpod-preferences.h
 * @see_also: #GtkWindow
 *
 * The #GpodPreferences is an instance #GtkWindow, and acts as
 * the settings window for the current #GtkApplication.
 **/

typedef struct _GpodPreferencesPrivate
{
    GpodWindow *window;
    GSettings *settings;

    GtkWidget *toolbar;
    GtkWidget *headerbar;
    GtkWidget *content;

    GpodPreferencesView *view;
} GpodPreferencesPrivate;

struct _GpodPreferences
{
    GtkWindow parent_instance;
};

G_DEFINE_TYPE_WITH_PRIVATE (GpodPreferences, gpod_preferences, GTK_TYPE_WINDOW)

/*
 * _on_key_press:
 * @widget: The original #GtkWidget that created the #G_CALLBACK
 * @event: The #GdkEventKey or the key that was pressed to start the callback
 * @user_data: User data set when the signal handler was connected, in this
 * instance it should be a #GtkWidget (#GpodPreferences).
 *
 * On a keypress from the user, check for a case in the switch that matchs the
 * keypress. If the keypress exists, run that code and return %TRUE. Otherwise
 * return %FALSE.
 *
 * Returns: #gboolean; %TRUE key press case found and code executed or %FALSE
 * no case found.
 */
static gboolean
_on_key_press (GtkWidget   *widget,
               GdkEventKey *event,
               gpointer     user_data)
{
    switch (event->keyval)
    {
        case GDK_KEY_Escape:
            gtk_widget_destroy (widget);
            return TRUE;

            break;

        case GDK_KEY_Q:
        case GDK_KEY_q:
            if (event->state & GDK_CONTROL_MASK)
            {
                gtk_widget_destroy (widget);
                return TRUE;
            }
            break;

        default:
            return FALSE;
    }

    return FALSE;
}

/*
 * _setup_page_general:
 * @object: An instance of #GpodPreferences
 *
 * Setups general page.
 */
void
_setup_page_general (GpodPreferences *self)
{
    GpodPreferencesPrivate *priv = gpod_preferences_get_instance_private (self);

    g_assert (GPOD_IS_PREFERENCES (self));

    gpod_preferences_view_add_page (priv->view, "general", "General");
    gpod_preferences_view_add_group (priv->view, "general", "example", "Example");

    GtkSwitch *widget1 = g_object_new (GTK_TYPE_SWITCH,
                                       "visible", TRUE,
                                       "valign", GTK_ALIGN_CENTER,
                                       NULL);

    g_settings_bind (priv->settings, "example-button",
                     GTK_WIDGET (widget1), "active",
                     G_SETTINGS_BIND_DEFAULT);

    gpod_preferences_view_add_option (priv->view, "general", "example", "Example GtkSwitch", "This is a switch example", GTK_WIDGET (widget1));



    GtkComboBoxText *widget2 = g_object_new (GTK_TYPE_COMBO_BOX_TEXT,
                                             "visible", TRUE,
                                             "valign", GTK_ALIGN_CENTER,
                                             NULL);

    gtk_combo_box_text_insert (widget2, 0, "0", "Example Combo Option 0");
    gtk_combo_box_text_insert (widget2, 1, "1", "Example Combo Option 1");
    gtk_combo_box_text_insert (widget2, 2, "2", "Example Combo Option 2");
    gtk_combo_box_text_insert (widget2, 3, "3", "Example Combo Option 3");

    g_settings_bind (priv->settings, "example-combo",
                     GTK_WIDGET (widget2), "active-id",
                     G_SETTINGS_BIND_DEFAULT);

    gpod_preferences_view_add_option (priv->view, "general", "example", "Example GtkComboBox", "This is a combo-box example", GTK_WIDGET (widget2));
}

/*
 * gpod_preferences_add_page:
 * @object: An instance of #GpodPreferences
 *
 * Setups all pages.
 */
void
_setup_pages (GpodPreferences *self)
{
    _setup_page_general (self);
}

/*
 * gpod_window_constructed:
 * @object: An instance of #GpodWindow
 *
 * The constructed virtual function sets first-run settings for the
 * #GpodWindow object.
 *
 * It sets:
 *  *
 */
static void
gpod_preferences_constructed (GObject *object)
{
    GpodPreferences *self = GPOD_PREFERENCES (object);
    GpodPreferencesPrivate *priv = gpod_preferences_get_instance_private (self);

    priv->view = gpod_preferences_view_new ();

    gtk_container_add (GTK_CONTAINER (priv->content),
                       GTK_WIDGET (priv->view));

    /* Setup Defaults */
    _setup_pages (self);

    G_OBJECT_CLASS (gpod_preferences_parent_class)->constructed (object);
}

/*
 * gpod_preferences_finalize:
 * @object: An instance of #GpodPreferences
 *
 * Does any last minute clean-ups when the class ends.
 */
static void
gpod_preferences_finalize (GObject *object)
{
    GpodPreferencesPrivate *priv = gpod_preferences_get_instance_private (GPOD_PREFERENCES (object));

    g_clear_object (&priv->settings);

    G_OBJECT_CLASS (gpod_preferences_parent_class)->finalize (object);
}

/*
 * gpod_preferences_init:
 * @self: An instance of #GpodPreferences
 *
 * Initializes various aspects of the window. Anything the window would need
 * before becoming presented should be done here.
 */
static void
gpod_preferences_init (GpodPreferences *self)
{
    GpodPreferencesPrivate *priv = gpod_preferences_get_instance_private (self);

    gtk_widget_init_template (GTK_WIDGET (self));

     /* Set window title */
    gtk_window_set_title (GTK_WINDOW (self), _("Preferences"));

    /* Set default window decorations (a.k.a GtkHeaderBar) */
    gtk_window_set_titlebar (GTK_WINDOW (self), priv->headerbar);
    gtk_container_add (GTK_CONTAINER (priv->headerbar), priv->toolbar);

    /* Connect escape key */
    g_signal_connect (G_OBJECT (self),
                      "key_press_event",
                      G_CALLBACK (_on_key_press),
                      self);

    /* Bind settings from GUI */
    priv->settings = g_settings_new (PACKAGE_ID);
}

/*
 * gpod_preferences_class_init:
 * @klass: An instance of #GpodPreferencesClass
 *
 * Initializes the various virtural functions in #GpodPreferencesClass.
 * In addtition it connects the various #GtkWidgets from the gresource file
 * using gtk_widget_class_set_template_from_resource().
 */
static void
gpod_preferences_class_init (GpodPreferencesClass *klass)
{
    GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
    GObjectClass *object_class = G_OBJECT_CLASS (klass);

    object_class->constructed = gpod_preferences_constructed;
    object_class->finalize = gpod_preferences_finalize;

    gtk_widget_class_set_template_from_resource (widget_class,
                                                 PACKAGE_RESOURCES "/ui/preferences.ui");

    gtk_widget_class_bind_template_child_private (widget_class, GpodPreferences, toolbar);
    gtk_widget_class_bind_template_child_private (widget_class, GpodPreferences, headerbar);
    gtk_widget_class_bind_template_child_private (widget_class, GpodPreferences, content);
}

/**
 * gpod_preferences_new:
 * @window: A #GtkApplicationWindow instance
 *
 * Creates a new #GpodPreferences, a sub-object of #GtkWindow.
 *
 * ## Usage
 * |[
 * GpodPreferences *preferences;
 *
 * preferences = gpod_preferences_new (window);
 * ]|
 *
 * When creating a gpod_preferences_new() a #GpodWindow will need
 * to be passed. This is because #GpodPreferences acts as a modal window
 * to the master window #GpodWindow attached to the
 * #GpodApplication.
 *
 * Returns: #GpodPreferences; a sub-object of #GtkWindow
 */
GpodPreferences *
gpod_preferences_new (GpodWindow *window)
{
    return g_object_new (GPOD_TYPE_PREFERENCES,
                         "transient-for", window,
                         NULL);
}
